"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AuthBearer = void 0;
const joi_1 = __importDefault(require("joi"));
const cm_libs_context_aware_error_1 = require("cm-libs-context-aware-error");
const intercall_1 = require("../intercall");
const auth_type_1 = require("./auth-type");
const auth_type_verify_1 = require("./auth-type-verify");
class AuthBearer {
    constructor(urlOrfnVerify) {
        if ('string' === typeof urlOrfnVerify) {
            this.identityUrl = urlOrfnVerify;
        }
        else {
            this.fnVerify = urlOrfnVerify;
        }
        if (!this.identityUrl && !this.fnVerify) {
            (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.PARAMETER_MISSING, 'Identity URL is missing', { urlOrfnVerify });
        }
    }
    static init(urlOrfnVerify) {
        this.self = new AuthBearer(urlOrfnVerify);
    }
    async verifyIdToken(authToken) {
        if (undefined === authToken?.info?.token) {
            (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.UNAUTHORISED, 'Id token is missing');
        }
        let _verifiedIdToken;
        if (this.fnVerify) {
            // Internal use by Identity service to verify idToken
            _verifiedIdToken = await this.fnVerify(authToken.info.token);
        }
        else {
            // Call Identity service to verify idToken, check blacklist, and return idToken info (e.g. rules, etc)
            _verifiedIdToken = await (0, intercall_1.callGraphQl)(this.identityUrl, `query PaidChain { verifyIdToken { appDomain, userId, userName, localeCode, rules, productId }}`, 'verifyIdToken', {
                authorization: authToken.info.token
            });
        }
        const { error, value } = joi_1.default.object(auth_type_verify_1.schemaVerifyIdToken).validate(_verifiedIdToken, { stripUnknown: true });
        const { rules, ...verifiedIdToken } = value;
        if (error) {
            (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.INVALID_FORMAT, error.message, {
                function: 'verifyIdToken',
                details: error.details,
                text: { input: error.details?.[0]?.context?.peers || error.details?.[0]?.context?.key }
            });
        }
        // Verification via internal function need to keep the rules to return to caller on the response message
        if (this.fnVerify) {
            authToken.info.authMetadata ??= {};
            authToken.info.authMetadata.rules = rules;
        }
        const ability = (0, auth_type_1.generateAbilityFromPackedRules)(rules);
        const idToken = {
            ...verifiedIdToken,
            ability
        };
        authToken.type = auth_type_1.AuthTokenType.Bearer;
        authToken.grant = true;
        authToken.info.auth = idToken;
        authToken.info.ability = ability;
        authToken.info.requestedBy = idToken.userId;
        authToken.info.requestedByName = idToken.userName;
        return authToken;
    }
    static get instance() {
        return this.self || (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.NOT_INITIALIZED, 'AuthBearer initialization is required');
    }
}
exports.AuthBearer = AuthBearer;
//# sourceMappingURL=auth-bearer.js.map